﻿unit Oblong.Temperature;

interface

uses
  System.ComponentModel, System.Drawing, System.Drawing.Design,
  Oblong.KelvinEditor, Oblong.TemperatureEditor;

const
  AbsoluteZeroC: Double = -273.15; //°C
  AbsoluteZeroK: Double = 0; //°C

type
  Temperature = class;

  /// A procedural type that underlies the TTemperature.OnInvalidTemp event.
  InvalidTempEventArgs = class(EventArgs)
  private
    FHandled: Boolean;
    Fkelvin: Double;
  public
    property Handled: Boolean read FHandled write FHandled;
    property kelvin: Double read FKelvin write Fkelvin;
  end;

  InvalidTempHandler = procedure(Sender: &Object; e: InvalidTempEventArgs) of object;

{$R 'Oblong.Temperature.bmp'}

  /// A class for converting temperatures from one scale to another.
  /// The TTemperature class is a non-visual component.
  [DefaultProperty('kelvin')]
  [DefaultEvent('InvalidTemp')]
  [Designer(TypeOf(TemperatureEditor))]
  [ToolboxBitmap(TypeOf(Temperature), 'Temperature.bmp')]
  Temperature = class(System.ComponentModel.Component)
  {$REGION 'Designer Managed Code'}
  strict private
    /// <summary>
    /// Required designer variable.
    /// </summary>
    Components: System.ComponentModel.Container;
    /// <summary>
    /// Required method for Designer support - do not modify
    /// the contents of this method with the code editor.
    /// </summary>
    procedure InitializeComponent;
  {$ENDREGION}
  strict private
    Fkelvin: Double;
    FInvalidTempHandler: InvalidTempHandler;
  strict protected
    /// <summary>
    /// Clean up any resources being used.
    /// </summary>
    procedure Dispose(Disposing: Boolean); override;
  public
    constructor Create; overload;
    constructor Create(Container: System.ComponentModel.IContainer); overload;
    /// Routine to signal InvalidTemp event
    procedure OnInvalidTemp(e: InvalidTempEventArgs); virtual;
    /// Property reader for Celsius and Centigrade properties
    function get_Celsius: Double;
    /// Property reader for Delisle property
    function get_Delisle: Double;
    /// Property reader for Fahrenheit property
    function get_Fahrenheit: Double;
    /// Property reader for kelvin property
    function get_kelvin: Double;
    /// Property reader for Newton property
    function get_Newton: Double;
    /// Property reader for Romer property
    function get_Romer: Double;
    /// Property reader for Reaumur property
    function get_Reaumur: Double;
    /// Property reader for Rankine property
    function get_Rankine: Double;
    /// Property writer for Celsius and Centigrade properties
    procedure set_Celsius(const Value: Double);
    /// Property writer for Fahrenheit property
    procedure set_Fahrenheit(const Value: Double);
    /// Property writer for kelvin property
    procedure set_kelvin(const Value: Double);
    /// Property writer for Newton property
    procedure set_Newton(const Value: Double);
    /// Property writer for Romer property
    procedure set_Romer(const Value: Double);
    /// Property writer for Reaumur property
    procedure set_Reaumur(const Value: Double);
    /// Property writer for Rankine property
    procedure set_Rankine(const Value: Double);
    /// Property writer for Delisle property
    procedure set_Delisle(const Value: Double);
  published
    /// Temperature in degrees Celsius (or Centigrade).
    /// Notes: Proposed (in a slightly different form) in 1742 by Swedish astronomer Anders Celsius.
    /// In 1948 officially renamed from Centigrade.
    /// The freezing point of water is 0 °C, the boiling point is 100 °C
    /// and absolute zero is at -273.15 °C.
    /// One degree Celsius equals one kelvin.
    /// Standard abbreviation is °C
    [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
    [Description('Temperature in degrees Celsius (°C)')]
    [Category('Temperature')]
    property Celsius: Double read get_Celsius write set_Celsius;
    /// Temperature in degrees Centigrade (or Celsius).
    /// Notes: Proposed (in a slightly different form) in 1742 by Swedish astronomer Anders Celsius.
    /// In 1948 Centigrade was officially renamed to Celsius.
    /// The freezing point of water is 0 °C, the boiling point is 100 °C
    /// and absolute zero is at -273.15 °C.
    /// One degree Centigrade equals one kelvin.
    /// Standard abbreviation is °C
    [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
    [Description('Temperature in degrees Centigrade (°C)')]
    [Category('Temperature')]
    property Centigrade: Double read get_Celsius write set_Celsius;
    /// Temperature in degrees Delisle.
    /// Notes: Invented in 1732 by French astronomer Joseph-Nicolas Delisle.
    /// It is similar to that of Réaumur.
    /// The freezing point of water is 150 °De, and the boiling point is 0 °De
    /// and absolute zero is at 559.725 °De.
    /// One degree Delisle equals −2/3 of a kelvin or degree Celsius.
    /// Standard abbreviation is °De
    [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
    [Description('Temperature in degrees Delisle (°De)')]
    [Category('Temperature')]
    property Delisle: Double read get_Delisle write set_Delisle;
    /// Temperature in degrees Fahrenheit.
    /// Notes: Proposed in 1708 by German physicist Gabriel Fahrenheit.
    /// The freezing point of water is 32 °F, the boiling point is 212 °F
    /// and absolute zero is -459.67 °F.
    /// One degree Fahrenheit equals 5/9th of a kelvin or degree Celsius.
    /// Standard abbreviation is °F
    [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
    [Description('Temperature in degrees Fahrenheit (°F)')]
    [Category('Temperature')]
    property Fahrenheit: Double read get_Fahrenheit write set_Fahrenheit;
    /// Temperature in kelvin units.
    /// Notes: Introduced in 1954 as the SI unit of temperature.
    /// Named after William Thomson, 1st Baron Kelvin.
    /// The freezing point of water is 273.15 K, the boiling point is 373.15 K
    /// and absolute zero (when molecular motion stops) is 0 K.
    /// One kelvin is the fraction 1/273.16 of the thermodynamic temperature of the triple point of water.
    /// One kelvin equals one degree Celsius.
    /// Standard abbreviation is K
    [DefaultValue(273.15)]
    [Description('Temperature in kelvin units (K)')]
    [Editor(TypeOf(KelvinEditor), TypeOf(UITypeEditor))]
    [Category('Temperature')]
    property kelvin: Double read get_kelvin write set_kelvin;
    /// Temperature in degrees Newton.
    /// Notes: Proposed around 1700 by Isaac Newton.
    /// The freezing point of water is 0 °N, the boiling point is 33 °N
    /// and absolute zero is -90.1395 °N.
    /// 1 degree Newton equals 100/33rd of a kelvin or a degree Celcius.
    /// Standard abbreviation is °N
    [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
    [Description('Temperature in degrees Newton (°N)')]
    [Category('Temperature')]
    property Newton: Double read get_Newton write set_Newton;
    /// Temperature in degrees Rankine.
    /// Notes: Proposed in 1859 by Scottish engineer and physicist William John Macquorn Rankine.
    /// The freezing point of water is 491.67 °R, the boiling point is 671.67 °R
    /// and absolute zero is 0 °R.
    /// One degree Rankine equals one degree Fahrenheit.
    /// Standard abbreviation is °R
    [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
    [Description('Temperature in degrees Rankine (°R)')]
    [Category('Temperature')]
    property Rankine: Double read get_Rankine write set_Rankine;
    /// Temperature in degrees Réaumur.
    /// Notes: Proposed in 1731 by René Antoine Ferchault de Réaumur.
    /// Also known as the "division octogesimale" or "octogesimal division".
    /// The freezing point of water is 0 °Ré, the boiling point 80 °Ré
    /// and absolute zero is -218.52 °Ré.
    /// One degree Reaumur is 1.25 degrees Celsius or kelvins.
    /// Standard abbreviation is °R, but °Ré allows unique identification
    [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
    [Description('Temperature in degrees Réaumur (°Ré)')]
    [Category('Temperature')]
    property Reaumur: Double read get_Reaumur write set_Reaumur;
    /// Temperature in degrees Rømer.
    /// Notes: Proposed in 1701 by Danish astronomer Ole Christensen Rømer.
    /// The freezing point of water is 7.5 °Rø, the boiling point 60 °Rø
    /// and absolute zero is -135.9038 °Rø.
    /// One degree Rømer equals 40/21st of a kelvin or degree Celsius.
    /// Standard abbreviation is °R, but °Rø allows unique identification
    [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
    [Description('Temperature in degrees Rømer (°Rø)')]
    [Category('Temperature')]
    property Romer: Double read get_Romer write set_Romer;
    /// OnInvalidTemp is fired when a temperature below absolute
    /// zero is assigned to any of the temperature properties
    [Description('Event that fires when a temperature below absolute zero is assigned')]
    property InvalidTemp: InvalidTempHandler add FInvalidTempHandler remove FInvalidTempHandler;
  end;

implementation

{$AUTOBOX ON}

{ Temperature }

resourcestring
  STempTooLow = 'Temperature less than absolute zero is invalid';

const
  RoundToDigits = 8;

{$REGION 'Windows Form Designer generated code'}
/// <summary>
/// Required method for Designer support - do not modify
/// the contents of this method with the code editor.
/// </summary>
procedure Temperature.InitializeComponent;
begin
end;
{$ENDREGION}

constructor Temperature.Create;
begin
  inherited Create;
  //
  // Required for Windows Form Designer support
  //
  InitializeComponent;
  //
  // TODO: Add any constructor code after InitializeComponent call
  //
  FKelvin := 273.15
end;

constructor Temperature.Create(Container: System.ComponentModel.IContainer);
begin
  inherited Create;
  //
  // Required for Windows Form Designer support
  //
  Container.Add(Self);
  InitializeComponent;
  //
  // TODO: Add any constructor code after InitializeComponent call
  //
  FKelvin := 273.15
end;

procedure Temperature.Dispose(Disposing: Boolean);
begin
  if Disposing then
  begin
    if Components <> nil then
      Components.Dispose();
  end;
  inherited Dispose(Disposing);
end;

function Temperature.get_Celsius: Double;
begin
  Result := Math.Round(Kelvin - 273.15, RoundToDigits)
end;

procedure Temperature.set_Celsius(const Value: Double);
begin
  Kelvin := Value + 273.15
end;

function Temperature.get_Delisle: Double;
begin
  Result := Math.Round((373.15 - Kelvin) * 3/2, RoundToDigits)
end;

procedure Temperature.set_Delisle(const Value: Double);
begin
  Kelvin := 373.15 - (Value * 2/3)
end;

function Temperature.get_Fahrenheit: Double;
begin
  Result := Math.Round((Kelvin * 1.8) - 459.67, RoundToDigits)
end;

procedure Temperature.set_Fahrenheit(const Value: Double);
begin
  Kelvin := (Value + 459.67) / 1.8
end;

function Temperature.get_kelvin: Double;
begin
  Result := Math.Round(Fkelvin, RoundToDigits)
end;

procedure Temperature.set_kelvin(const Value: Double);
var
  e: InvalidTempEventArgs;
begin
  if Value < AbsoluteZeroK then
  begin
    e := InvalidTempEventArgs.Create;
    e.kelvin := Value;
    OnInvalidTemp(e)
  end
  else
    FKelvin := Value;
end;

function Temperature.get_Newton: Double;
begin
  Result := Math.Round((Kelvin - 273.15) * 33/100, RoundToDigits)
end;

procedure Temperature.set_Newton(const Value: Double);
begin
  Kelvin := (Value * 100/33) + 273.15
end;

function Temperature.get_Rankine: Double;
begin
  Result := Math.Round(Kelvin * 9 / 5, RoundToDigits)
end;

procedure Temperature.set_Rankine(const Value: Double);
begin
  Kelvin := Value * 5 / 9
end;

function Temperature.get_Reaumur: Double;
begin
  Result := Math.Round((Kelvin - 273.15) * 4 / 5, RoundToDigits)
end;

procedure Temperature.set_Reaumur(const Value: Double);
begin
  Kelvin := (Value * 5 / 4) + 273.15
end;

function Temperature.get_Romer: Double;
begin
  Result := Math.Round(((Kelvin - 273.15) * 21/40) + 7.5, RoundToDigits)
end;

procedure Temperature.set_Romer(const Value: Double);
begin
 Kelvin := ((Value - 7.5) * 40/21) + 273.15
end;

procedure Temperature.OnInvalidTemp(e: InvalidTempEventArgs);
begin
  e.Handled := False;
  if Assigned(FInvalidTempHandler) then
    FInvalidTempHandler(Self, e);
  if not e.Handled then
    raise ArgumentException.Create(STempTooLow)
end;

end.
